# Generated by Django 4.2.6 on 2024-07-19 02:03

from django.db import migrations
import tinymce.models


class Migration(migrations.Migration):

    dependencies = [
        ("djf_surveys", "0013_survey_success_page_content"),
    ]

    operations = [
        migrations.AlterField(
            model_name="survey",
            name="success_page_content",
            field=tinymce.models.HTMLField(
                blank=True, null=True, verbose_name="Success Page Content"
            ),
        ),
    ]
