<div data-te-datepicker-init class="datepicker">
    <input type="{{ widget.type }}" name="{{ widget.name }}"{% if widget.value != None %}
        value="{{ widget.value }}"{% endif %}{% include "django/forms/widgets/attrs.html" %}
        data-te-datepicker-toggle-ref
        data-te-datepicker-toggle-button-ref/>
</div>
